
var deadzone = 0.15;

var target : Transform;
var reticle : Transform;

var cam : Camera;
var cursor_pic : Transform;

private var throttle = 0.0;
private var cam_ray = Ray(Vector3.zero, Vector3.forward);

// The camera doesnt start following the target immediately but waits for a split second to avoid too much waving around.
private var lockCameraTimer = 0.0;

// The current move direction in x-z
private var moveDirection = Vector3.zero;
private var forward = Vector3.zero;
private var right = Vector3.zero;
// The current vertical speed
private var verticalSpeed = 0.0;
// The current x-z move speed
private var moveSpeed = 0.0;

// The last collision flags returned from controller.Move
private var collisionFlags : CollisionFlags; 

private var isControllable = true;

private var puppet : ShipModel;

function Awake ()
{
	moveDirection = transform.TransformDirection(Vector3.forward);
    forward = transform.TransformDirection(Vector3.forward);
	right = Vector3(forward.z, 0, -forward.x);
}

function Start()
{
	puppet = GetComponent(ShipModel);

    Screen.showCursor = false;
    Screen.lockCursor = true;
}

// This next function responds to the "HidePlayer" message by hiding the player. 
// The message is also 'replied to' by identically-named functions in the collision-handling scripts.
// - Used by the LevelStatus script when the level completed animation is triggered.

function HidePlayer()
{
	GameObject.Find("rootJoint").GetComponent(SkinnedMeshRenderer).enabled = false; // stop rendering the player.
	isControllable = false;	// disable player controls.
}

// This is a complementary function to the above. We don't use it in the tutorial, but it's included for
// the sake of completeness. (I like orthogonal APIs; so sue me!)

function ShowPlayer()
{
	GameObject.Find("rootJoint").GetComponent(SkinnedMeshRenderer).enabled = true; // start rendering the player again.
	isControllable = true;	// allow player to control the character again.
}


function clamp(value, min, max) {
    if(value < min){ value = min; }
    else if(value > max){ value = max; }
    return value;
}

function FixedUpdate() {
	if (Input.GetButton ("Accelerate"))
	{
        var delta = Input.GetAxis("Accelerate");
        var throttle = puppet.GetThrottle();
        if(throttle < 1.0 && delta > 0.0) {
          throttle += Time.fixedDeltaTime * delta;
        }
        else if(throttle > 0.0 && delta < 0.0) {
          throttle += Time.fixedDeltaTime * delta;
        }
        puppet.SetThrottle(throttle);
	}

    var cursor = Vector3(0,0,0);
    cursor.x = (Input.mousePosition.x - Screen.width/2) / (Screen.width/2);
    cursor.y = (Input.mousePosition.y - Screen.height/2) / (Screen.height/2);

    puppet.SetCursor(cursor);

    var turnx = 0.0;
    var turny = 0.0;


    cam_ray = cam.ScreenPointToRay (Vector3(Input.mousePosition.x,Input.mousePosition.y,0));

    // FIXME : just set this to weapon range sometime...
    var aim_point = cam_ray.GetPoint(1000.0);
    puppet.SetAim(new Ray(transform.position, aim_point));
    //puppet.SetAim(cam_ray);

    // clamp the cursor to the deadzone
    if(cursor.x > deadzone) {
        turnx = (cursor.x - deadzone) / (1.0 - deadzone);
    }
    else if (cursor.x < -deadzone) {
        turnx = (cursor.x + deadzone) / (1.0 - deadzone);
    }
    else {
        turnx = 0.0;
    }
    if(cursor.y > deadzone) {
        turny = (cursor.y - deadzone) / (1.0 - deadzone);
    }
    else if (cursor.y < -deadzone) {
        turny = (cursor.y + deadzone) / (1.0 - deadzone);
    }
    else {
        turny = 0.0;
    }
    
    var turn_direction = Vector3 (-turny, turnx, Input.GetAxis("Roll"));
    //var turn_direction = Vector3 (Input.GetAxis("Vertical"), Input.GetAxis("Vertical"), Input.GetAxis("Roll"));
    puppet.Turn(turn_direction);

    var camera_bounce = 0.02;

    //cam.transform.position.x = -turnx * camera_bounciness;
    cam.transform.localPosition = new Vector3(turnx * camera_bounce, 0.1 + turny * camera_bounce, -0.3);
}


function Fire() {
    //var aim_point = puppet.GetAim().GetPoint(1000.0);
    var aim_point = cam_ray.GetPoint(1000.0);
    BroadcastMessage("Pewpew", aim_point, SendMessageOptions.DontRequireReceiver);
}


function Update() {
    forward = transform.TransformDirection(Vector3.forward);
	
	if (!isControllable)
	{
		// kill all inputs if not controllable.
		Input.ResetInputAxes();
	}

    cursor_pic.position.x = Input.mousePosition.x / Screen.width;
    cursor_pic.position.y = Input.mousePosition.y / Screen.height;

    if (Input.GetButton("Fire1")) {
        Fire();
    }
    /*
	// Calculate actual motion
	var movement = moveDirection * moveSpeed + Vector3 (0, verticalSpeed, 0) + inAirVelocity;
	movement *= Time.deltaTime;
	
	// Move the controller
	var controller : CharacterController = GetComponent(CharacterController);
	collisionFlags = controller.Move(movement);
	
	transform.rotation = Quaternion.LookRotation(moveDirection);
    */
    //var reticle_pos = cam.WorldToScreenPoint(target.position);
    //reticle.transform.position.x = reticle_pos.x / Screen.width;
    //reticle.transform.position.y = reticle_pos.y / Screen.height;

    FixedUpdate();
}

function OnControllerColliderHit (hit : ControllerColliderHit )
{
//	Debug.DrawRay(hit.point, hit.normal);
	if (hit.moveDirection.y > 0.01) 
		return;
	wallJumpContactNormal = hit.normal;
}

function GetSpeed () {
	return moveSpeed;
}

function GetThrottle() {
    return throttle;
}

function GetDirection () {
	return moveDirection;
}


function GetLockCameraTimer () 
{
	return lockCameraTimer;
}

function IsMoving ()  : boolean
{
	return Mathf.Abs(Input.GetAxisRaw("Vertical")) + Mathf.Abs(Input.GetAxisRaw("Horizontal")) > 0.5;
}


@script RequireComponent(ShipModel)
@script AddComponentMenu("Oolong/Ship (Controller -- Mouse)")
