
var team : String;

// The speed when walking
var impulse = 10.0;
var inertia = 0.3;

var yaw = 25.0;
var pitch = 25.0;
var roll = 35.0;

var hitPoints = 100;

var explosionType : GameObject;

private var throttle = 0.0;
private var realImpulse = impulse;
private var cursor = Vector3(0.5,0.5,0.0);
private var aim_ray = Ray(Vector3.zero, Vector3.forward);
private var aim_position = Vector3.zero;

private var shieldsUp = false;
private var shield_cooldown = 0.0;

// The camera doesnt start following the target immediately but waits for a split second to avoid too much waving around.
private var lockCameraTimer = 0.0;

// The current move direction in x-z
private var moveDirection = Vector3.zero;
private var forward = Vector3.zero;
private var right = Vector3.zero;
// The current vertical speed
private var verticalSpeed = 0.0;
// The current x-z move speed
private var moveSpeed = 0.0;

// The last collision flags returned from controller.Move
private var collisionFlags : CollisionFlags; 

// Are we moving backwards (This locks the camera to not do a 180 degree spin)
private var movingBack = false;
// Is the user pressing any keys?
private var isMoving = false;

private var isControllable = true;

static private var cameraObject : Camera;
private var shield_mesh : Transform;

function Awake ()
{
	moveDirection = transform.TransformDirection(Vector3.forward);
    forward = transform.TransformDirection(Vector3.forward);
	right = Vector3(forward.z, 0, -forward.x);

    cameraObject = FindObjectOfType(Camera);
	shield_mesh = transform.Find("shield_fighter");
	
	SetInertia(inertia);
}

// This next function responds to the "HidePlayer" message by hiding the player. 
// The message is also 'replied to' by identically-named functions in the collision-handling scripts.
// - Used by the LevelStatus script when the level completed animation is triggered.

function HidePlayer()
{
	GameObject.Find("rootJoint").GetComponent(SkinnedMeshRenderer).enabled = false; // stop rendering the player.
	isControllable = false;	// disable player controls.
}

// This is a complementary function to the above. We don't use it in the tutorial, but it's included for
// the sake of completeness. (I like orthogonal APIs; so sue me!)

function ShowPlayer()
{
	GameObject.Find("rootJoint").GetComponent(SkinnedMeshRenderer).enabled = true; // start rendering the player again.
	isControllable = true;	// allow player to control the character again.
}

function Shields() {
	shieldsUp = true;
	if(shield_mesh != null) shield_mesh.renderer.enabled = true;
	var shield_delay = 1.0;
	//shield_cooldown = shield_delay;
	//Debug.Log("shields are "+ shield_cooldown);
}

function ShieldsDown() {
	shieldsUp = false;
	if(shield_mesh != null) shield_mesh.renderer.enabled = false;
	shield_cooldown = 0;
}

function Thrust(scale) {    
    // get the body
    rigidbody.AddForce(forward * scale * realImpulse * Time.fixedDeltaTime);
}

function Turn(direction) {
    var xrot = pitch * direction.x * Time.fixedDeltaTime;
    var yrot = yaw * direction.y * Time.fixedDeltaTime;
    var zrot = roll * direction.z * Time.fixedDeltaTime;
    var xQuaternion = Quaternion.AngleAxis (xrot, Vector3.right); 
    var yQuaternion = Quaternion.AngleAxis (yrot, Vector3.up); 
    var zQuaternion = Quaternion.AngleAxis (zrot, Vector3.forward); 

    //var deltaRotation = Quaternion.Euler(xQuaternion);
    var totalRotation = xQuaternion * yQuaternion * zQuaternion;
    //var totalRotation = Vector3(direction.x * pitch, direction.y * yaw, direction.z * roll);
    //totalRotation *= Time.fixedDeltaTime;

    rigidbody.MoveRotation(rigidbody.rotation * totalRotation);
    //Debug.Log("Rotation is: " + rigidbody.rotation);
    //transform.Rotate(totalRotation);
    //rigidbody.rotation = transform.rotation;
}


function clamp(value, min, max) {
    if(value < min){ value = min; }
    else if(value > max){ value = max; }
    return value;
}

function turnTo(direction : Vector3) {
    var destination_rot = Quaternion.LookRotation(direction);
    //var difference = destination_rot - transform.rotation;

    var str = Mathf.Min (yaw * Time.deltaTime, 1); 

    transform.rotation = Quaternion.Slerp(transform.rotation, destination_rot, str);
    //RotateTowards(transform.rotation, destination_rot, str, 1.0);
}


function turnTo2D(aim_point : Vector3) {
    // FIXME : for 2D stuff we should probably just do our own 
    // math that's probably faster
    var targetDir = aim_point - transform.position;
	var destination_rot = Quaternion.LookRotation(targetDir, Vector3.forward*-1);
    //var difference = destination_rot - transform.rotation;

	var step = Mathf.Min(yaw * Time.deltaTime, 1);
	//transform.Rotate(0,step,0);
	transform.rotation = Quaternion.Slerp(transform.rotation, destination_rot, step);
	//transform.rotation.eulerAngles.y = Mathf.Lerp(transform.rotation.eulerAngles.z, destination_angle, str);
    //RotateTowards(transform.rotation, destination_rot, str, 1.0);
}

function avoid2D(aim_point : Vector3) {
    // FIXME : for 2D stuff we should probably just do our own 
    // math that's probably faster
    var targetDir = transform.position - aim_point;
	var destination_rot = Quaternion.LookRotation(targetDir, Vector3.forward*-1);
    //var difference = destination_rot - transform.rotation;

	var step = Mathf.Min(yaw * Time.deltaTime, 1);
	//transform.Rotate(0,step,0);
	transform.rotation = Quaternion.Slerp(transform.rotation, destination_rot, step);
	//transform.rotation.eulerAngles.y = Mathf.Lerp(transform.rotation.eulerAngles.z, destination_angle, str);
    //RotateTowards(transform.rotation, destination_rot, str, 1.0);
}




function FixedUpdate() {
    //Thrust(throttle);

    //transform.position = rigidbody.position;
    //transform.localRotation = rigidbody.rotation;
}

function Special() {
}

function Fire(aim_point : Vector3) {
    if (aim_point == null) {
        aim_point = aim_ray.GetPoint(1000.0);
    }

    BroadcastMessage("Pewpew", aim_point, SendMessageOptions.DontRequireReceiver);
}


function OnDamage (amount) {
    hitPoints -= amount;
}

/*
function OnCollisionEnter(collision : Collision) {
    var bullet : Bullet = collision.gameObject.GetComponent(Bullet);
    var base_damage = bullet.damage;
    OnDamage(base_damage);
    Debug.Log("HP is now " + hitPoints);
}
*/

function OnTriggerEnter(collider : Collider) {
    var bullet : Bullet = collider.gameObject.GetComponent(Bullet);
	if(bullet != null && bullet.team != team && !shieldsUp) {
		var base_damage = bullet.damage;
		BroadcastMessage("OnDamage",base_damage,SendMessageOptions.DontRequireReceiver);
		//Debug.Log("HP is now " + hitPoints);
	}
}

function Update() {
    forward = transform.TransformDirection(Vector3.forward);
    //FixedUpdate();
	if( shield_cooldown > 0 ){ shield_cooldown -= Time.deltaTime; Debug.Log("shields are "+ shield_cooldown);}
	if( shield_cooldown < 0 ) ShieldsDown();

    if (hitPoints <= 0 ) { Kill(); }
}

function Kill() {
    //transform.DetachChildren();
    var trails = transform.GetComponentsInChildren(TrailRenderer); 
    for(var trail : TrailRenderer in trails) {
        trail.transform.parent=null;
        trail.autodestruct = true;
    }

	//var director = FindObjectOfType(Director);
	//director.OnDeath(this);
    boom = Instantiate(explosionType, transform.position, transform.rotation);
	cameraObject.GetComponent(Shake).DoEffect();
    Destroy(gameObject);
}

function GetSpeed () {
	return moveSpeed;
}

function SetThrottle(t : float) {
    t = clamp(t, 0.0, 1.0);
    throttle = t;
}

function GetThrottle() {
    return throttle;
}

function SetCursor(c : Vector3) {
    c.x = clamp(c.x, -1.0, 1.0);
    c.y = clamp(c.y, -1.0, 1.0);

    cursor = c;
}

function GetCursor() {
    return cursor;
}

function SetAim(r : Ray) {
    aim_ray = r;
}

/*
function AimAt(aim_point : Vector3) {
    forward = transform.TransformDirection(Vector3.forward);
    transform.LookAt(Vector3.RotateTowards(forward, aim_point, firing_arc, 1000.0));
}
*/

function GetAim() {
    return aim_ray;
}

function GetAimPos() {
    return aim_position;
}

function SetAimPos(value : Vector3) {
    aim_position = value;
}

function GetDirection () {
	return moveDirection;
}

function IsMovingBackwards () {
	return movingBack;
}

function GetLockCameraTimer () 
{
	return lockCameraTimer;
}

function IsMoving ()  : boolean
{
	return Mathf.Abs(Input.GetAxisRaw("Vertical")) + Mathf.Abs(Input.GetAxisRaw("Horizontal")) > 0.5;
}

function GetInertia() : float {
	//var inertia = 1.0 - rigidbody.drag;
	return inertia;
}

function SetInertia(value : float) {
    var new_drag = 1.0 - value;
	rigidbody.drag = new_drag;
	realImpulse = impulse + (new_drag * impulse);
	inertia = value;
}


function Reset ()
{
	gameObject.tag = "Player";
}
// Require a character controller to be attached to the same game object
@script RequireComponent(Rigidbody)
@script AddComponentMenu("Oolong/Ship (Model)")
