// The speed when walking
var impulse = 10.0;
var inertia = 0.3;

var yaw = 25.0;
var pitch = 25.0;
var roll = 35.0;

private var throttle = 0.0;
private var realImpulse = impulse;
private var cursor = Vector3(0.5,0.5,0.0);
private var aim_ray = Ray(Vector3.zero, Vector3.forward);
private var aim_position = Vector3.zero;

// The camera doesnt start following the target immediately but waits for a split second to avoid too much waving around.
private var lockCameraTimer = 0.0;

private var forward = Vector3.zero;
private var right = Vector3.zero;

// The last collision flags returned from controller.Move
private var collisionFlags : CollisionFlags; 

private var isControllable = true;

function Awake ()
{
    forward = transform.TransformDirection(Vector3.forward);
	right = Vector3(forward.z, 0, -forward.x);
	
	SetInertia(inertia);
}


function Thrust(scale) {    
    // get the body
    rigidbody.AddForce(forward * scale * realImpulse * Time.fixedDeltaTime);
}


function Turn(direction) {
    var xrot = pitch * direction.x * Time.fixedDeltaTime;
    var yrot = yaw * direction.y * Time.fixedDeltaTime;
    var zrot = roll * direction.z * Time.fixedDeltaTime;
    var xQuaternion = Quaternion.AngleAxis (xrot, Vector3.right); 
    var yQuaternion = Quaternion.AngleAxis (yrot, Vector3.up); 
    var zQuaternion = Quaternion.AngleAxis (zrot, Vector3.forward); 

    //var deltaRotation = Quaternion.Euler(xQuaternion);
    var totalRotation = xQuaternion * yQuaternion * zQuaternion;
    //var totalRotation = Vector3(direction.x * pitch, direction.y * yaw, direction.z * roll);
    //totalRotation *= Time.fixedDeltaTime;

    rigidbody.MoveRotation(rigidbody.rotation * totalRotation);
    //Debug.Log("Rotation is: " + rigidbody.rotation);
    //transform.Rotate(totalRotation);
    //rigidbody.rotation = transform.rotation;
}


function clamp(value, min, max) {
    if(value < min){ value = min; }
    else if(value > max){ value = max; }
    return value;
}

function turnTo(direction : Vector3) {
    var destination_rot = Quaternion.LookRotation(direction);
    //var difference = destination_rot - transform.rotation;

    var str = Mathf.Min (yaw * Time.deltaTime, 1); 

    transform.rotation = Quaternion.Slerp(transform.rotation, destination_rot, str);
    //RotateTowards(transform.rotation, destination_rot, str, 1.0);
}


function turnTo2D(aim_point : Vector3) {
    // FIXME : for 2D stuff we should probably just do our own 
    // math that's probably faster
    var targetDir = aim_point - transform.position;
    
    /*
    right = transform.TransformDirection(Vector3.right);
    var turnDir = 1;
    if(Vector3.Dot(right, targetDir) > 0) turnDir = 1;
    else turnDir = -1;
	
    transform.Rotate(0,yaw * Time.fixedDeltaTime * turnDir,0);
    */

	var destination_rot = Quaternion.LookRotation(targetDir, Vector3.forward*-1);
    //var difference = destination_rot - transform.rotation;

	var step = Mathf.Min(yaw * Time.deltaTime, 1);
	transform.rotation = Quaternion.Slerp(transform.rotation, destination_rot, step);
	//transform.rotation.eulerAngles.y = Mathf.Lerp(transform.rotation.eulerAngles.z, destination_angle, str);
    //RotateTowards(transform.rotation, destination_rot, str, 1.0);
}


function FixedUpdate() {
    Thrust(throttle);
    //transform.position = rigidbody.position;
    //transform.localRotation = rigidbody.rotation;
}


function Update() {
    forward = transform.TransformDirection(Vector3.forward);
}

function SetThrottle(t : float) {
    t = clamp(t, 0.0, 1.0);
    throttle = t;
}

function GetThrottle() {
    return throttle;
}

function GetInertia() : float {
	//var inertia = 1.0 - rigidbody.drag;
	return inertia;
}

function SetInertia(value : float) {
    var new_drag = 1.0 - value;
	rigidbody.drag = new_drag;
	realImpulse = impulse + (new_drag * impulse);
	inertia = value;
}

function SetCursor(c : Vector3) {
    c.x = clamp(c.x, -1.0, 1.0);
    c.y = clamp(c.y, -1.0, 1.0);

    cursor = c;
}

function GetCursor() {
    return cursor;
}

function GetAimPos() {
    return aim_position;
}

function SetAimPos(value : Vector3) {
    aim_position = value;
}


@script RequireComponent(Rigidbody)
@script AddComponentMenu("Physics/Steerable")
